<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:xalan="http://xml.apache.org/xalan"
                xmlns:tpl="http://software.in2p3.fr/lavoisier/application.xsd"
                xmlns="http://software.in2p3.fr/lavoisier/application.xsd"
                exclude-result-prefixes="xalan tpl">
    <xsl:output method="xml" indent="yes" xalan:indent-amount="4"/>
    <xsl:strip-space elements="*"/>

    <xsl:template match="tpl:element-recursive">
        <xsl:param name="count"/>
        <xsl:variable name="ancestor">
            <xsl:choose>
                <!-- WORKAROUND: @*[local-name()='ancestor'] BECAUSE attribute::ancestor DOES NOT WORK -->
                <xsl:when test="@*[local-name()='ancestor']"><xsl:value-of select="@*[local-name()='ancestor']"/></xsl:when>
                <xsl:otherwise>1</xsl:otherwise>
            </xsl:choose>
        </xsl:variable>
        <xsl:choose>
            <xsl:when test="$count=''">
                <xsl:choose>
                    <xsl:when test="$ancestor = 0">
                        <xsl:apply-templates select="//tpl:*[tpl:element-recursive]">
                            <xsl:with-param name="count" select="@count - 1"/>
                        </xsl:apply-templates>
                    </xsl:when>
                    <xsl:otherwise>
                        <xsl:apply-templates select="ancestor::tpl:*[position() = $ancestor]">
                            <xsl:with-param name="count" select="@count - 1"/>
                        </xsl:apply-templates>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:when>
            <xsl:when test="$count > 1">
                <xsl:choose>
                    <xsl:when test="$ancestor = 0">
                        <xsl:apply-templates select="//tpl:*[tpl:element-recursive]">
                            <xsl:with-param name="count" select="$count - 1"/>
                        </xsl:apply-templates>
                    </xsl:when>
                    <xsl:otherwise>
                        <xsl:apply-templates select="ancestor::tpl:*[position() = $ancestor]">
                            <xsl:with-param name="count" select="$count - 1"/>
                        </xsl:apply-templates>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:when>
        </xsl:choose>
    </xsl:template>

    <xsl:template match="tpl:elements | tpl:elements-ignore">
        <xsl:param name="path">
            <xsl:choose>
                <xsl:when test="starts-with(@path, '/')">
                    <xsl:value-of select="concat(substring-after(@path,'/'), '/')"/>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:value-of select="concat(@path, '/')"/>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:param>
        <xsl:element name="{translate(local-name(),'s','')}" namespace="http://software.in2p3.fr/lavoisier/application.xsd">
            <xsl:variable name="predicate-position" select="string-length(substring-before($path,'['))+1"/>
            <xsl:variable name="slash-position" select="string-length(substring-before($path,'/'))+1"/>
            <xsl:variable name="has-predicate" select="$slash-position>$predicate-position and $predicate-position>1"/>
            <xsl:variable name="tag">
                <xsl:choose>
                    <xsl:when test="$has-predicate"><xsl:value-of select="substring($path, 0, $predicate-position)"/></xsl:when>
                    <xsl:otherwise><xsl:value-of select="substring-before($path, '/')"/></xsl:otherwise>
                </xsl:choose>
            </xsl:variable>
            <xsl:if test="$tag=''">
                <xsl:message terminate="yes">Attribute @path does not support recursive XPath: //</xsl:message>
            </xsl:if>
            <xsl:if test="contains($tag,'::')">
                <xsl:message terminate="yes">Attribute @path does not support XPath axis: <xsl:value-of select="substring-before($tag,'::')"/></xsl:message>
            </xsl:if>
            <xsl:if test="$tag!='*'">
                <xsl:attribute name="in"><xsl:value-of select="$tag"/></xsl:attribute>
            </xsl:if>

            <xsl:if test="$has-predicate">
                <xsl:attribute name="if"><xsl:value-of select="substring-before(substring($path,$predicate-position+1), ']')"/></xsl:attribute>
            </xsl:if>

            <xsl:variable name="remains">
                <xsl:choose>
                    <xsl:when test="$has-predicate"><xsl:value-of select="substring-after($path, ']/')"/></xsl:when>
                    <xsl:otherwise><xsl:value-of select="substring-after($path, '/')"/></xsl:otherwise>
                </xsl:choose>
            </xsl:variable>
            <xsl:choose>
                <xsl:when test="$remains != ''">
                    <xsl:apply-templates select="self::tpl:*">
                        <xsl:with-param name="path" select="$remains"/>
                    </xsl:apply-templates>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:copy-of select="@attributes"/>
                    <xsl:apply-templates/>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:element>
    </xsl:template>

    <xsl:template match="tpl:element[@namespace-ignore='true'][@out]">
        <xsl:message terminate="yes">Attribute @namespace-ignore is not needed when attribute @out is set, please remove it.</xsl:message>
    </xsl:template>

    <xsl:template match="*">
        <xsl:param name="count"/>
        <xsl:copy><xsl:copy-of select="@*"/>
            <xsl:apply-templates>
                <xsl:with-param name="count" select="$count"/>
            </xsl:apply-templates>
        </xsl:copy>
    </xsl:template>
</xsl:stylesheet>