<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns="http://www.w3.org/TR/xpath" xmlns:x="http://www.w3.org/TR/xpath">
    <xsl:output method="xml"/>

    <xsl:template match="/x:*/x:*/x:*[not(self::x:AbsoluteLocationPath)]">
        <xsl:message terminate="yes">This adaptor only supports absolute path</xsl:message>
    </xsl:template>
    <xsl:template match="x:Predicate[ancestor::x:Predicate]">
        <xsl:message terminate="yes">This adaptor does not support nested predicates</xsl:message>
    </xsl:template>
    <xsl:template match="x:AbsoluteLocationPath[ancestor::x:Predicate]">
        <xsl:message terminate="yes">This adaptor does not support absolute path in predicates</xsl:message>
    </xsl:template>
    <xsl:template match="x:Function">
        <xsl:message terminate="yes">This adaptor does not support functions</xsl:message>
    </xsl:template>
    <xsl:template match="x:AllNodeStep">
        <xsl:message terminate="yes">This adaptor does not support unnamed node</xsl:message>
    </xsl:template>
    <xsl:template match="x:CommentNodeStep | x:ProcessingInstructionNodeStep | x:UnionExpr | x:VariableReference">
        <xsl:message terminate="yes">This adaptor does not support <xsl:value-of select="local-name()"/></xsl:message>
    </xsl:template>

    <xsl:template match="x:AbsoluteLocationPath">
        <xsl:copy>
            <xsl:apply-templates/>
        </xsl:copy>
    </xsl:template>
    <xsl:template match="x:RelativeLocationPath">
        <xsl:apply-templates/>
    </xsl:template>

    <xsl:template match="x:AbsoluteLocationPath/x:NameStep[@axis='child']" priority="1">
        <Child>
            <xsl:copy-of select="@prefix | @localName"/>
            <xsl:apply-templates/>
        </Child>
    </xsl:template>
    <xsl:template match="x:RelativeLocationPath/x:NameStep[@axis='attribute']" priority="1">
        <Attribute>
            <xsl:copy-of select="@localName"/>
            <xsl:apply-templates/>
        </Attribute>
    </xsl:template>
    <xsl:template match="x:NameStep">
        <xsl:message terminate="yes">This adaptor does not support axis: <xsl:value-of select="@axis"/></xsl:message>
    </xsl:template>

    <xsl:template match="x:Predicate">
        <xsl:apply-templates select="x:*[last()]"/>
    </xsl:template>

    <xsl:template match="x:AndExpr | x:OrExpr" priority="1">
        <xsl:copy>
            <xsl:apply-templates select="(x:PathExpr|x:AndExpr|x:OrExpr|x:NotExpr)[last()]/preceding-sibling::x:*[1]"/>
            <xsl:apply-templates select="x:*[last()]"/>
        </xsl:copy>
    </xsl:template>
    <xsl:template match="x:EqualityExpr[@symbol = '=']" priority="1">
        <Equals>
            <xsl:apply-templates select="preceding-sibling::x:*[1]"/>
            <xsl:apply-templates select="x:*"/>
        </Equals>
    </xsl:template>
    <xsl:template match="x:*[@symbol != '=']">
        <xsl:message terminate="yes">This adaptor does not support operator: <xsl:value-of select="@symbol"/></xsl:message>
    </xsl:template>

    <xsl:template match="x:Literal">
        <xsl:copy>
            <xsl:value-of select="@s"/>
        </xsl:copy>
    </xsl:template>
    <xsl:template match="x:Number">
        <xsl:copy>
            <xsl:value-of select="@i"/>
        </xsl:copy>
    </xsl:template>
</xsl:stylesheet>